/* GdbDisplayController.h created by ovidiu on Wed 19-Mar-1997 */

#import <Foundation/NSObject.h>
#import <GuiDisplayProvider_Protocol.h>

@class NSMutableArray;
@class NSBrowser;
@class NSMatrix;

@class Stack;

@interface GdbDisplayController : NSObject <GuiDisplayProvider2>
{
  id gdbManager;

  id stackBrowser;

  id runButton;
  id stopButton;
  id quitButton;
  id execCmdMatrix;
  id lineMatrix;
  id selectionMatrix;

  DebuggerState debuggerState;

  id objectToBeNotified;
  SEL selectorToBeInvoked;
  NSMutableArray* objectsToBeNotified;

  Stack* stack;
  int currentFrame;
}

- (void)display:sender;
- (void)close;

- (void)setButtonsConformingToState:(DebuggerState)state;
- (void)disableAllButtons;

  /*  Send a command to GDB. Since the command is execute asyncronously, `object'
  will be notified by invoking `selector' on it with the output string as the
  first argument and a GdbOutputType value as the second argument when GDB
  finishes the execution. Use this method for all GDB commands for which you
  are waiting for output results. */
- (void)executeGDBCommand:(NSString*)command
                 annotate:(BOOL)flag
             notifyObject:object
                 selector:(SEL)selector;

  /* Outlet methods */
- (void)run:sender;
- (void)interrupt:sender;
- (void)quit:sender;
- (void)continue:sender;
- (void)step:sender;
- (void)finish:sender;
- (void)next:sender;
- (void)breakAt:sender;
- (void)runUntil:sender;
- (void)print:sender;
- (void)printIndirect:sender;
- (void)showStackFrameWindow:sender;

  /* Delegate methods of stack browser */
- (void)browser:(NSBrowser*)sender
  createRowsForColumn:(int)column
       inMatrix:(NSMatrix*)matrix;

- (int)frameNumberForBrowserCell:(int)row;
- (int)browserCellForFrame:(int)frameNumber;

  /* Changing the appearence of stack browser */
- (void)stackChanged;
- (void)updateCurrentFrame;

  /* Other methods */
- (id)gdbManager;
- (id)stackBrowser;
- (void)selectInFile:(NSString*)file startLine:(int)sl endLine:(int)el;

- (void)invalidateCurrentFrameVariables;

@end
